/*
 *      funGUI.cpp 
 *      Funlight main window
 */

#include <ZApplication.h>
#include <ZMainWidget.h>
#include <ezxutilcst.h>
#include <ZPushButton.h>
#include <ZKeyObject.h>
#include <qlabel.h>
#include <qslider.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtextview.h>
#include <ZMessageBox.h>
#include <SetupSetManager.h>

#include "funGUI.h"
#include "HelpWindow.h"
#include "funlight.h"

Funlight funobj = Funlight(0x0A00);
extern int fl_fb;
extern char sfsname[256],sfdname[256],sfdsname[256];
unsigned int fbbr;

SETUP_SET_Manager mgr(AM_Global::AppsAlarmClock, false);

funGUI::funGUI()
	: ZMainWidget( false, NULL, NULL, 0 )
{
	sttd = false;
	mgr.getSettingInfo("SYS_SYSTEM_SET", "Brightness", fbbr, (unsigned)25); // getting brightness from system settings

	lang[0].name = "English";
	lang[0].start = "Start";
	lang[0].stop = "Stop";
	lang[0].sens = "Sensitivity: %d";
	lang[0].brightness = "Brightness of backlight";
	lang[0].smooth = "Smooth flashing";
	lang[0].startd = "Start daemon";
	lang[0].stopd = "Stop daemon";
	lang[0].dq = "Start daemon with this scheme?";
	lang[0].yes = "Yes";
	lang[0].no = "No";
	lang[0].help = "Help";
	lang[0].about = "About";
	lang[0].s.s1 = "Funky";
	lang[0].s.s2 = "Pulse";
	lang[0].s.s3 = "Rhytmic";
	lang[0].s.s4 = "Flicker";
	lang[0].s.s5 = "Energized";
	lang[0].s.s6 = "Beats";
	lang[0].s.s7 = "Mania";
	lang[0].s.s8 = "Alien";
	lang[0].s.s9 = "Elemental";
	lang[0].s.s10 = "Sonic";

	lang[1].name = QString::fromUtf8("Русский");
	lang[1].start = QString::fromUtf8("Старт");
	lang[1].stop = QString::fromUtf8("Стоп");
	lang[1].sens = "Чувствительность: %d";
	lang[1].brightness = QString::fromUtf8("Яркость подсветки");
	lang[1].smooth = QString::fromUtf8("Плавное мигание");
	lang[1].startd = QString::fromUtf8("Запуск демона");
	lang[1].stopd = QString::fromUtf8("Остановка демона");
	lang[1].dq = QString::fromUtf8("Запустить демон с этой схемой?");
	lang[1].yes = QString::fromUtf8("Да");
	lang[1].no = QString::fromUtf8("Нет");
	lang[1].help = QString::fromUtf8("Помощь");
	lang[1].about = QString::fromUtf8("О программе");
	lang[1].s.s1 = QString::fromUtf8("Супер");
	lang[1].s.s2 = QString::fromUtf8("Пульс");
	lang[1].s.s3 = QString::fromUtf8("Ритм");
	lang[1].s.s4 = QString::fromUtf8("Мерцание");
	lang[1].s.s5 = QString::fromUtf8("Энергия");
	lang[1].s.s6 = QString::fromUtf8("Биения");
	lang[1].s.s7 = QString::fromUtf8("Мания");
	lang[1].s.s8 = QString::fromUtf8("Чужой");
	lang[1].s.s9 = QString::fromUtf8("Элементы");
	lang[1].s.s10 = QString::fromUtf8("Акустика");
	

  strcat(sfsname, ".conf");
	sfile = open(sfsname, O_RDONLY | O_NONBLOCK);
  if(sfile != -1)
  {
    read(sfile, &settings, 8);
    if(settings.lang != 1)
      settings.lang = 0;
    if(settings.sensval > 9)
      settings.sensval = 6;
    if(settings.brval > 100)
      settings.brval = fbbr;
  }
  else
  {
    settings.lang = 0;
    settings.sensval = 6;
    settings.brval = fbbr;
    settings.brset = false;
    settings.smset = false;
  }

	// Titlebar
	title = new QLabel(" Funlight", this);
	title->setScaledContents(true);
	setTitleBarWidget(title);

  // Help button
	helpbtn = new ZPushButton(title);
	helpbtn->setText("?");
	helpbtn->setGeometry(215, 0, 56, 25);
	
	helpmenu = new QPopupMenu(this);
	helpmenu->insertItem(lang[settings.lang].help, this, SLOT(h0()), 0, 0);
	helpmenu->insertItem(lang[settings.lang].about, this, SLOT(h1()), 0, 1);
	
	helpbtn->setPopup(helpmenu);

	// CST widget.
	cst = new UTIL_CST(this, lang[settings.lang].start);
	setCSTWidget(cst);

  // Exit button
	exitbtn = cst->getRightBtn();
	connect(exitbtn, SIGNAL(clicked()), this, SLOT(exitSlot()));

  // Menu button
	lbtn = cst->getLeftBtn();
	eng = new QRadioButton(lang[0].name,0);
	lang1 = new QRadioButton(lang[1].name,0);
	if(settings.lang == 1)
    lang1->setChecked(true);
  else
    eng->setChecked(true);
	connect(eng, SIGNAL(clicked()), this, SLOT(tolang0()));
	connect(lang1, SIGNAL(clicked()), this, SLOT(tolang1()));
	
	style = new QPopupMenu(0,lang[settings.lang].startd);
	style->insertItem(lang[settings.lang].s.s1, this, SLOT(m0()), 0, 0);
	style->insertItem(lang[settings.lang].s.s2, this, SLOT(m1()), 0, 1);
	style->insertItem(lang[settings.lang].s.s3, this, SLOT(m2()), 0, 2);
	style->insertItem(lang[settings.lang].s.s4, this, SLOT(m3()), 0, 3);
	style->insertItem(lang[settings.lang].s.s5, this, SLOT(m4()), 0, 4);
	style->insertItem(lang[settings.lang].s.s6, this, SLOT(m5()), 0, 5);
	style->insertItem(lang[settings.lang].s.s7, this, SLOT(m6()), 0, 6);
	style->insertItem(lang[settings.lang].s.s8, this, SLOT(m7()), 0, 7);
	style->insertItem(lang[settings.lang].s.s9, this, SLOT(m8()), 0, 8);
	style->insertItem(lang[settings.lang].s.s10, this, SLOT(m9()), 0, 9);

	menu = new QPopupMenu(this);
	menu->insertItem(eng);
	menu->insertItem(lang1);
	menu->insertSeparator();
	menu->insertItem(lang[settings.lang].startd, style, 3);
	menu->insertItem(lang[settings.lang].stopd, this, SLOT(stopd()), 0, 4);
	
	lbtn->setPopup(menu);

	// Start/stop button
	btn = cst->getMidBtn();
	connect(btn, SIGNAL(clicked()), this, SLOT(funStartStop()));

  // Content label
	contentLabel = new QLabel("", this);
	contentLabel->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
	contentLabel->setAlignment(Qt::AlignLeft);
	setContentWidget(contentLabel);
	
	// Sensitivity label
	sprintf(labeltext, lang[settings.lang].sens, settings.sensval);
	sensLabel = new QLabel(QString::fromUtf8(labeltext), contentLabel);
	sensLabel->setFixedSize(220,32);
	sensLabel->move(10,10);
	sensLabel->setAlignment(Qt::AlignLeft);

	// Sensitivity slider
	slider1 = new QSlider(0, 9, 1, settings.sensval, Qt::Horizontal, contentLabel);
	slider1->setFixedSize(220, 32);
	slider1->move(10,42);
  slider1->setTickmarks(QSlider::Below);
  slider1->update();

  // "min"
  minLabel = new QLabel(" min", contentLabel);
  minLabel->setAlignment(Qt::AlignLeft);
  minLabel->move(0,74);
  minLabel->setFixedSize(80, 32);

  // "max"
  maxLabel = new QLabel("max ", contentLabel);
  maxLabel->setAlignment(Qt::AlignRight);
  maxLabel->setFixedSize(80, 32);
  maxLabel->move(160,74);

  // Brighness checkbox
  chBox1 = new QCheckBox(lang[settings.lang].brightness, contentLabel);
  chBox1->setChecked(settings.brset);
  chBox1->setFixedSize(240, 32);
  chBox1->move(10,106);
  connect(chBox1, SIGNAL(clicked()), this, SLOT(brSlot()));

  // Brightness slider
  slider2 = new QSlider(0, 100, 5, settings.brval, Qt::Horizontal, contentLabel);
	slider2->setFixedSize(220, 32);
	slider2->move(10,138);
  slider2->setTickmarks(QSlider::Below);
  slider2->update();
  if(!settings.brset)
    slider2->setDisabled(true);

  // Smooth brighness checkbox
  chBox2 = new QCheckBox(lang[settings.lang].smooth, contentLabel);
  chBox2->setChecked(settings.smset);
  chBox2->setFixedSize(240, 32);
  chBox2->move(10,170);
  connect(chBox2, SIGNAL(clicked()), this, SLOT(smSlot()));

  // Keypad control
  
  keyCenter = new ZKeyObject(this);
  keyCenter->addKey(EZX_KEY_CENTER_SELECT);
  connect(keyCenter, SIGNAL(keyPressed(int)), this, SLOT(funStartStop()));

  keyRight = new ZKeyObject(this);
  keyRight->addKey(EZX_KEY_RIGHT);
  connect(keyRight, SIGNAL( keyPressed(int)), this, SLOT(krSlot()));

  keyLeft = new ZKeyObject(this);
  keyLeft->addKey(EZX_KEY_LEFT);
  connect(keyLeft, SIGNAL(keyPressed(int)), this, SLOT(klSlot()));
  
  timer = new QTimer();
  timer->start(100);
  connect(timer, SIGNAL(timeout()), this, SLOT(chSens()));
  
  
}

funGUI::~funGUI()
{

}

void funGUI::exitSlot()
{
  settings.sensval = slider1->value();
  settings.brval = slider2->value();
  settings.brset = chBox1->isChecked();
  settings.smset = chBox2->isChecked();
  sfile = open(sfsname, O_CREAT | O_WRONLY);
  write(sfile, &settings, 8);
  close(sfile);
  funobj.stop();
  ioctl(fl_fb, FBIOSETBRIGHTNESS, fbbr);
  qApp->quit();
}

void funGUI::startd(int itemId)
{
  char cmdline[256];
  char cmdline1[256];
  int res;
  sprintf(cmdline, sfdname, itemId);
  system(cmdline);
  ZMessageBox *msgbox = new ZMessageBox(this, 0, lang[settings.lang].dq, lang[settings.lang].yes, lang[settings.lang].no);
  res = msgbox->exec();
  if(res==0)
  {
    system("kill `ps -eo pid,ucmd | grep funlightd` > /dev/null 2> /dev/null");
    sprintf(cmdline1, sfdsname, itemId);
    system(cmdline1);
  }
  delete msgbox;
}

void funGUI::stopd()
{
  system("kill `ps -eo pid,ucmd | grep funlightd` > /dev/null 2> /dev/null");
}

void funGUI::m0()
{
  startd(0);
}
void funGUI::m1()
{
  startd(1);
}
void funGUI::m2()
{
  startd(2);
}
void funGUI::m3()
{
  startd(3);
}
void funGUI::m4()
{
  startd(4);
}
void funGUI::m5()
{
  startd(5);
}
void funGUI::m6()
{
  startd(6);
}
void funGUI::m7()
{
  startd(7);
}
void funGUI::m8()
{
  startd(8);
}
void funGUI::m9()
{
  startd(9);
}

void funGUI::h0()
{
  HelpWindow *help = new HelpWindow(settings.lang);
  help->show();
}

void funGUI::h1()
{
  ZMessageBox *msgbox = new ZMessageBox(this, 0, QString::fromUtf8(" <u>Funlight - Rhytm Lights</u><br /> Version: 1.1<br /> Author: kycb-kycb<br /> E-mail: kycb-kycb@my-moto.ru<br /><br /> http://forum.motofan.ru/<br /> http://my-moto.ru/"), "OK");
  msgbox->exec();
  delete msgbox;
}

void funGUI::funStartStop()
{
  extern int fl_fbbr;
  
  if(!sttd)
  {
    chSens();
    if(chBox1->isChecked())
      fl_fbbr = slider2->value();
    else
      fl_fbbr = fbbr;
    funobj.start();
    sttd = true;
    btn->setText(lang[settings.lang].stop);
    menu->setItemEnabled(3, false);
  }
  else
  {
    funobj.stop();
    ioctl(fl_fb, FBIOSETBRIGHTNESS, fbbr);
    sttd = false;
    btn->setText(lang[settings.lang].start);
    menu->setItemEnabled(3, true);
  }
}

void funGUI::brSlot()
{
  if(chBox1->isChecked())
  {
    slider2->setEnabled(true);
    if(chBox2->isChecked())
      chBox2->setChecked(false);
  }
  else
    slider2->setDisabled(true);
}

void funGUI::smSlot()
{
  if(chBox2->isChecked())
  {
    chBox1->setChecked(false);
    slider2->setDisabled(true);
  }
}

void funGUI::krSlot()
{
  if(slider1->value() < 9)
    slider1->setValue(slider1->value()+1);
}

void funGUI::klSlot()
{
  if(slider1->value() > 0)
    slider1->setValue(slider1->value()-1);
}

void funGUI::chSens()
{
  sprintf(labeltext, lang[settings.lang].sens, slider1->value());
  sensLabel->setText(QString::fromUtf8(labeltext));
  if(chBox2->isChecked())
    funobj.setSmooth(true);
  else
    funobj.setSmooth(false);

  // Setting sensitivity level
  switch(slider1->value())
  {
    case 0:
      funobj.setSens(0x6747);
    break;
    case 1:
      funobj.setSens(0x42A1);
    break;
    case 2:
      funobj.setSens(0x2AFD);
    break;        
    case 3:
      funobj.setSens(0x1BBC);
    break;        
    case 4:
      funobj.setSens(0x11E4);
    break;        
    case 5:
      funobj.setSens(0x0B8B);
    break;
    case 6:
      funobj.setSens(0x0772);
    break;
    case 7:
      funobj.setSens(0x04CE);
    break;
    case 8:
      funobj.setSens(0x0319);
    break;
    case 9:
      funobj.setSens(0x0200);
    break;
    default:
      funobj.setSens(0x0772);
  }
}

void funGUI::tolang0()
{
  lang1->setChecked(false);
  eng->setChecked(true);
  changeLang(0);
}

void funGUI::tolang1()
{
  lang1->setChecked(true);
  eng->setChecked(false);
  changeLang(1);
}

void funGUI::changeLang(int langn)
{
  if(langn != settings.lang)
  {
    settings.lang = langn;
    if(!sttd)
      btn->setText(lang[settings.lang].start);
    else
      btn->setText(lang[settings.lang].stop);
    chBox1->setText(lang[settings.lang].brightness);
    chBox2->setText(lang[settings.lang].smooth);
  
    menu->changeItem(lang[settings.lang].startd, 3);
    menu->changeItem(lang[settings.lang].stopd, 4);
    style->changeItem(lang[settings.lang].s.s1, 0);
  	style->changeItem(lang[settings.lang].s.s2, 1);
  	style->changeItem(lang[settings.lang].s.s3, 2);
  	style->changeItem(lang[settings.lang].s.s4, 3);
  	style->changeItem(lang[settings.lang].s.s5, 4);
  	style->changeItem(lang[settings.lang].s.s6, 5);
  	style->changeItem(lang[settings.lang].s.s7, 6);
  	style->changeItem(lang[settings.lang].s.s8, 7);
  	style->changeItem(lang[settings.lang].s.s9, 8);
  	style->changeItem(lang[settings.lang].s.s10, 9);
  	style->update();
  	menu->update();
  	helpmenu->changeItem(lang[settings.lang].help, 0);
  	helpmenu->changeItem(lang[settings.lang].about, 1);
  	helpmenu->update();
  }
}
