/*
 *      funlight.cpp
 *      Funlight class routine
 *      Author: kycb-kycb <mailto:kycb-kycb@my-moto.ru> 
 */

#include "funlight.h"

bool fl_run,fl_smooth;
short fl_sens;
int fl_keylight,fl_fb,fl_fbbr;

ZAudioNormalDev fl_mic = ZAudioNormalDev(SOUNDM_AUDIO_MICINPUT, 8000, 1, 2);

Funlight::Funlight(short sens)
{
  fl_mic.close();
  fl_sens = sens;
  fl_keylight = open("/dev/keylight", O_RDONLY | O_NONBLOCK);
  fl_fb = open("/dev/fb0", O_RDONLY);
}

Funlight::~Funlight()
{
  if(runned())
    stop();
  close(fl_fb);
  close(fl_keylight);
}

void Funlight::start()
{
  if(!runned())
  {
    fl_run = true;
    fl_mic.open();
    pthread_create(&fun_t, NULL, fun, NULL);
    ioctl(fl_fb, FBIOBLANK, 0);
    ioctl(fl_fb, FBIOSETBKLIGHT, BKLIGHT_OFF);
    ioctl(fl_keylight, KEYLIGHT_MAIN_OFF, 0);
  }
}

void Funlight::stop()
{
  fl_run = false;
  pthread_join(fun_t, NULL);
  fl_mic.close();
  ioctl(fl_keylight, KEYLIGHT_MAIN_OFF, 0);
  ioctl(fl_fb, FBIOBLANK, 0);
  ioctl(fl_fb, FBIOSETBKLIGHT, BKLIGHT_ON);
}

bool Funlight::runned()
{
  return fl_run;
}

void Funlight::setSens(short sens)
{
  fl_sens = sens;
}

void Funlight::setSmooth(bool sm)
{
  fl_smooth = sm;
}

void* Funlight::fun(void*) 
{
float l,brs;
int desc,i;
short buff[192],tbuff[256];
bool prf,isl=false,iskl=false;

  while(fl_run)
  {
    l=desc=0;
    prf=false;
    fl_mic.read(buff, 192);
    for(i=0;i<192;i++)
    {
      if(buff[i] < 0)
      {
        buff[i] = 0-buff[i];
        if(!prf)
        {
          prf=true;
          desc++;
        }
      }
      else
        prf=false;

      l += buff[i]/192;
    }
    l = l*(100/desc);

    if(l>fl_sens)
    {
      if(fl_smooth)
      {
        brs = (1-(fl_sens/l))*100;
        fl_fbbr = (int)brs;
      }
      ioctl(fl_fb, FBIOSETBRIGHTNESS, fl_fbbr);

      if(desc>=35)
      {
        if(iskl)
        ioctl(fl_keylight, KEYLIGHT_MAIN_OFF, 0);
      }
      else
        if(!iskl)
        {
          ioctl(fl_keylight, KEYLIGHT_MAIN_ON, 0);
          iskl = true;
        }
        else
          iskl = false;

      if(!isl)
      {
        ioctl(fl_fb, FBIOBLANK, 0);
        ioctl(fl_fb, FBIOSETBKLIGHT, BKLIGHT_ON);
        isl = true;
        fl_mic.read(tbuff, 256);
      }
    }

    if(l<=fl_sens)
    {
      if(!isl)
      {
        ioctl(fl_keylight, KEYLIGHT_MAIN_OFF, 0);
        ioctl(fl_fb, FBIOSETBKLIGHT, BKLIGHT_OFF);
      }
      else if(fl_smooth)
        ioctl(fl_fb, FBIOSETBRIGHTNESS, 0);
      isl = false;
    }
  }
  return 0;
}
