/*
 *       main.cpp
 *       Daemon funlightd
 *       Author: kycb-kycb <mailto:kycb-kycb@my-moto.ru> 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <linux/fb.h>
#include <fcntl.h>
#include <string.h>
#include <SetupSetManager.h>

#define KEYLIGHT_MAIN_ON    0xf0
#define KEYLIGHT_MAIN_OFF   0xf2
#define FBIOSETBKLIGHT      0x4619
#define FBIOGETBKLIGHT      0x461A
#define FBIOSETBRIGHTNESS   0x461B
#define FBIOGETBRIGHTNESS   0x461C
#define BKLIGHT_OFF         0
#define BKLIGHT_ON          1

extern "C" {
extern int UTIL_GetIncomingCallStatus();
}

char *argv0;
int fb,kl,ffun;
unsigned int brcurrent = 0;

// schemes offset table (First 80 bytes in schemes file)
struct funoffset
{
  unsigned int offset;
  unsigned int size; // size is set in words
};

// scheme word (4 bytes)
struct funword
{
  unsigned short msec;
  char screen;
  bool key;
};

// engine
void funshow(int scheme, bool demo = false)
{
  int i;
  funoffset foffset[10];
  bool iskl=false,isl=true,isbr=false;

  funword buff;
  
  fb = open("/dev/fb0", O_RDONLY);
  kl = open("/dev/keylight", O_RDONLY | O_NONBLOCK);
  ffun = open(argv0, O_RDONLY);
  
  if(ffun != -1 && fb != -1 && kl != -1) 
  {
    SETUP_SET_Manager mgr(AM_Global::AppsAlarmClock, false);
    mgr.getSettingInfo("SYS_SYSTEM_SET", "Brightness", brcurrent, (unsigned)25);
    read(ffun, &foffset, 80);

    if(demo)
      printf("Showing demo\n");
    else
      printf("Running daemon\n");

    for(;;)
    {
      lseek(ffun, foffset[scheme].offset, SEEK_SET);

      for(i=0;i<foffset[scheme].size;i++)
      {
        if(UTIL_GetIncomingCallStatus() || demo)
        {
          read(ffun, &buff, 4);
          if(buff.key)
          {
            ioctl(kl, KEYLIGHT_MAIN_ON, 0);
            iskl = true;
          }
          else
          {
            ioctl(kl, KEYLIGHT_MAIN_OFF, 0);
            iskl = false;
          }

          if(buff.screen > 0)
          {
            ioctl(fb, FBIOBLANK, 0);
            ioctl(fb, FBIOSETBKLIGHT, BKLIGHT_ON);
            isl = true;
            if(buff.screen > 1)
            {
              ioctl(fb, FBIOSETBRIGHTNESS, buff.screen);
              isbr = true;
            }
            else
            {
              ioctl(fb, FBIOSETBRIGHTNESS, brcurrent);
              isbr = false;
            }
          }
          else
          {
            ioctl(fb, FBIOSETBKLIGHT, BKLIGHT_OFF);
            isl = false;
          }
          usleep(buff.msec*1000);
        }
        else
        {
          if(!isl)
          {
            ioctl(fb, FBIOSETBKLIGHT, BKLIGHT_ON);
            isl = true;
          }
          if(isbr)
          {
            ioctl(fb, FBIOSETBRIGHTNESS, brcurrent);
            isbr = false;
          }
          if(iskl)
          {
            ioctl(kl, KEYLIGHT_MAIN_OFF, 0);
            iskl = false;
          }
          sleep(1);
          if(i>0)
            i = foffset[scheme].size;
          else
            i--;
        }
      }
      if(demo)
        break;
    }
    ioctl(fb, FBIOSETBKLIGHT, BKLIGHT_ON);
    ioctl(fb, FBIOSETBRIGHTNESS, brcurrent);
    ioctl(kl, KEYLIGHT_MAIN_OFF, 0);

    close(ffun);
    close(fb);
    close(kl);
    printf("Done\n");
  }
  else
  {
    if(fb == -1)
      printf("Can't open /dev/fb0\n");
    if(kl == -1)
      printf("Can't open /dev/keylight\n");
    if(ffun == -1)
      printf("Can't open schemes file\n");
  }
}

void sigexit(int)
{
  printf("Exiting\n");
  ioctl(fb, FBIOSETBKLIGHT, BKLIGHT_ON);
  ioctl(fb, FBIOSETBRIGHTNESS, brcurrent);
  ioctl(kl, KEYLIGHT_MAIN_OFF, 0);
  close(ffun);
  close(fb);
  close(kl);
  exit(0);
}

int main(int argc, char *argv[]) {

int arg_s;

  signal(SIGTERM, sigexit);
  signal(SIGINT, sigexit);

  if(argc > 1)
  {
    arg_s = argv[1][0];
    arg_s += -48;
    if(arg_s < 0 || arg_s > 9)
    {
      printf("Usage:\n    %s [scheme] [d]\n    %s --help\n    %s -h\n", argv[0], argv[0], argv[0]);
      printf(" sheme       - work scheme (0-9, 0 default)\n");
      printf("             Schemes:\n              0: Funky\n              1: Pulse\n              2: Rhytmic\n              3: Flicker\n              4: Energized\n              5: Beats\n              6: Mania\n              7: Alien\n              8: Elemental\n              9: Sonic\n");
      printf(" d           - show demo if set\n");
      printf(" --help, -h  - display this help\n");
      return 0;
    }
  }
		else
		  arg_s = 0;
		argv0 = argv[0];
		strcat(argv0, ".sch");

		if(argc > 2)
    {
		    funshow(arg_s, true);
		    return 0;
		}
		else
		{
      funshow(arg_s);
    }

    return 0;
}
